package net.esper.view;

import net.esper.view.std.*;
import net.esper.view.ext.*;
import net.esper.view.window.*;
import net.esper.view.stat.*;

/**
 * Enum for all build-in views.
 */
public enum ViewEnum
{
    /**
     * Length window.
     */
    LENGTH_WINDOW("win", "length", LengthWindowViewFactory.class, null),

    /**
     * Length batch window.
     */
    LENGTH_BATCH("win", "length_batch", LengthBatchViewFactory.class, null),

    /**
     * Time window.
     */
    TIME_WINDOW("win", "time", TimeWindowViewFactory.class, null),

    /**
     * Time batch.
     */
    TIME_BATCH("win","time_batch", TimeBatchViewFactory.class, null),

    /**
     * Externally timed window.
     */
    EXT_TIMED_WINDOW("win", "ext_timed", ExternallyTimedWindowViewFactory.class, null),

    /**
     * Size view.
     */
    SIZE("std", "size", SizeViewFactory.class, null),

    /**
     * Last event.
     */
    LAST_EVENT("std", "lastevent", LastElementViewFactory.class, null),

    /**
     * Unique.
     */
    UNIQUE_BY_PROPERTY("std", "unique", UniqueByPropertyViewFactory.class, null),

    /**
     * Group-by merge.
     */
    GROUP_MERGE("std", "merge", MergeViewFactory.class, null),

    /**
     * Group-by.
     */
    GROUP_PROPERTY("std", "groupby", GroupByViewFactory.class, GROUP_MERGE),

    /**
     * Univariate statistics.
     */
    UNIVARIATE_STATISTICS("stat", "uni", UnivariateStatisticsViewFactory.class, null),

    /**
     * Weighted avg.
     */
    WEIGHTED_AVERAGE("stat", "weighted_avg", WeightedAverageViewFactory.class, null),

    /**
     * Correlation.
     */
    CORRELATION("stat", "correl", CorrelationViewFactory.class, null),

    /**
     * Linest.
     */
    REGRESSION_LINEST("stat", "linest", RegressionLinestViewFactory.class, null),

    /**
     * Cubes.
     */
    MULTIDIM_VIEW("stat", "cube", MultiDimStatsViewFactory.class, null),

    /**
     * Sorted window.
     */
    SORT_WINDOW("ext", "sort", SortWindowViewFactory.class, null);


    private final String namespace;
    private final String name;
    private final Class factoryClass;
    private final ViewEnum mergeView;

    ViewEnum(String namespace, String name, Class factoryClass, ViewEnum mergeView)
    {
        this.namespace = namespace;
        this.name = name;
        this.factoryClass = factoryClass;
        this.mergeView = mergeView;
    }

    /**
     * Returns namespace that the object belongs to.
     * @return namespace
     */
    public String getNamespace()
    {
        return namespace;
    }

    /**
     * Returns name of the view that can be used to reference the view in a view expression.
     * @return short name of view
     */
    public String getName()
    {
        return name;
    }

    /**
     * Returns the enumeration value of the view for merging the data generated by another view.
     * @return view enum for the merge view
     */
    public ViewEnum getMergeView()
    {
        return mergeView;
    }

    /**
     * Returns a view's factory class.
     * @return class of view factory
     */
    public Class getFactoryClass()
    {
        return factoryClass;
    }

    /**
     * Returns the view enumeration value given the name of the view.
     * @param namespace is the namespace name of the view
     * @param name is the short name of the view as used in view expressions
     * @return view enumeration value, or null if no such view name is among the enumerated values
     */
    public static ViewEnum forName(String namespace, String name)
    {
        for (ViewEnum viewEnum : ViewEnum.values())
        {
            if ((viewEnum.namespace.equals(namespace)) && (viewEnum.name.equals(name)))
            {
                return viewEnum;
            }
        }

        return null;
    }
}
